@@ -26,6 +26,7 @@
 
 enum {
     TCP,
+    TCP_PI,
     RTU
 };
 
@@ -43,10 +44,12 @@ int main(int argc, char*argv[])
     if (argc > 1) {
         if (strcmp(argv[1], "tcp") == 0) {
             use_backend = TCP;
+        } else if (strcmp(argv[1], "tcppi") == 0) {
+            use_backend = TCP_PI;
         } else if (strcmp(argv[1], "rtu") == 0) {
             use_backend = RTU;
         } else {
-            printf("Usage:\n  %s [tcp|rtu] - Modbus server for unit testing\n\n", argv[0]);
+            printf("Usage:\n  %s [tcp|tcppi|rtu] - Modbus server for unit testing\n\n", argv[0]);
             return -1;
         }
     } else {
@@ -57,6 +60,9 @@ int main(int argc, char*argv[])
     if (use_backend == TCP) {
         ctx = modbus_new_tcp("127.0.0.1", 1502);
         query = malloc(MODBUS_TCP_MAX_ADU_LENGTH);
+    } else if (use_backend == TCP_PI) {
+        ctx = modbus_new_tcp_pi("::0", "1502");
+        query = malloc(MODBUS_TCP_MAX_ADU_LENGTH);
     } else {
         ctx = modbus_new_rtu("/dev/ttyUSB0", 115200, 'N', 8, 1);
         modbus_set_slave(ctx, SERVER_ID);
@@ -65,13 +71,12 @@ int main(int argc, char*argv[])
     header_length = modbus_get_header_length(ctx);
 
     modbus_set_debug(ctx, TRUE);
-    modbus_set_error_recovery(ctx, TRUE);
 
     mb_mapping = modbus_mapping_new(
-        UT_BITS_ADDRESS + UT_BITS_NB_POINTS,
-        UT_INPUT_BITS_ADDRESS + UT_INPUT_BITS_NB_POINTS,
-        UT_REGISTERS_ADDRESS + UT_REGISTERS_NB_POINTS,
-        UT_INPUT_REGISTERS_ADDRESS + UT_INPUT_REGISTERS_NB_POINTS);
+        UT_BITS_ADDRESS + UT_BITS_NB,
+        UT_INPUT_BITS_ADDRESS + UT_INPUT_BITS_NB,
+        UT_REGISTERS_ADDRESS + UT_REGISTERS_NB,
+        UT_INPUT_REGISTERS_ADDRESS + UT_INPUT_REGISTERS_NB);
     if (mb_mapping == NULL) {
         fprintf(stderr, "Failed to allocate the mapping: %s\n",
                 modbus_strerror(errno));
@@ -84,11 +89,11 @@ int main(int argc, char*argv[])
 
     /** INPUT STATUS **/
     modbus_set_bits_from_bytes(mb_mapping->tab_input_bits,
-                               UT_INPUT_BITS_ADDRESS, UT_INPUT_BITS_NB_POINTS,
+                               UT_INPUT_BITS_ADDRESS, UT_INPUT_BITS_NB,
                                UT_INPUT_BITS_TAB);
 
     /** INPUT REGISTERS **/
-    for (i=0; i < UT_INPUT_REGISTERS_NB_POINTS; i++) {
+    for (i=0; i < UT_INPUT_REGISTERS_NB; i++) {
         mb_mapping->tab_input_registers[UT_INPUT_REGISTERS_ADDRESS+i] =
             UT_INPUT_REGISTERS_TAB[i];;
     }
@@ -96,6 +101,9 @@ int main(int argc, char*argv[])
     if (use_backend == TCP) {
         socket = modbus_tcp_listen(ctx, 1);
         modbus_tcp_accept(ctx, &socket);
+    } else if (use_backend == TCP_PI) {
+        socket = modbus_tcp_pi_listen(ctx, 1);
+        modbus_tcp_pi_accept(ctx, &socket);
     } else {
         rc = modbus_connect(ctx);
         if (rc == -1) {
@@ -106,7 +114,7 @@ int main(int argc, char*argv[])
     }
 
     for (;;) {
-        rc = modbus_receive(ctx, -1, query);
+        rc = modbus_receive(ctx, query);
         if (rc == -1) {
             /* Connection closed by the client or error */
             break;
@@ -115,12 +123,13 @@ int main(int argc, char*argv[])
         /* Read holding registers */
         if (query[header_length] == 0x03) {
             if (MODBUS_GET_INT16_FROM_INT8(query, header_length + 3)
-                == UT_REGISTERS_NB_POINTS_SPECIAL) {
+                == UT_REGISTERS_NB_SPECIAL) {
                 printf("Set an incorrect number of values\n");
                 MODBUS_SET_INT16_TO_INT8(query, header_length + 3,
-                                         UT_REGISTERS_NB_POINTS);
+                                         UT_REGISTERS_NB_SPECIAL - 1);
             } else if (MODBUS_GET_INT16_FROM_INT8(query, header_length + 1)
                 == UT_REGISTERS_ADDRESS_SPECIAL) {
+                printf("Reply to this special register address by an exception\n");
                 modbus_reply_exception(ctx, query,
                                        MODBUS_EXCEPTION_SLAVE_OR_SERVER_BUSY);
                 continue;
